// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_SCPL_IO_WIN32_WAITABLEEVENT_H
#define HU_VARADIISTVAN_SCPL_IO_WIN32_WAITABLEEVENT_H
//------------------------------------------------------------------------------

#include "Event.h"
#include "EventFailable.h"

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

/**
 * An event which can be waited for.
 */
class WaitableEvent : public EventFailable, public Event
{
public:
    /**
     * Construct the event.
     */
    WaitableEvent(Waiter* waiter = 0);

    /**
     * Check if the event is fired or not. If so, the fired status
     * will be cleared.
     */
    bool check();
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline WaitableEvent::WaitableEvent(Waiter* waiter) :
    EventFailable(std::make_unique<EventFactory>()),
    Event(static_cast<EventFailable&>(*this))
{
    if (waiter!=0) addTo(*waiter);
}

//------------------------------------------------------------------------------

inline bool WaitableEvent::check()
{
    if (isFired()) {
        clear();
        return true;
    } else {
        return false;
    }
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_SCPL_IO_WIN32_WAITABLEEVENT_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
